
#define USE_HIFRCLOCK
/* If the macrodefinition above is removed or commented out, */
/* measured time accuracy is only up to 0.01s.               */
/* Windows code is not tested yet.                           */

#ifdef __linux__  /* Linux */
#ifdef USE_HIFRCLOCK
#include <sys/time.h>
#define __USE_POSIX199309
#include <time.h>
#else
#include <unistd.h>
#include <sys/times.h>
#endif
#else             /* Windows */
#ifdef USE_HIFRCLOCK
#include <profileapi.h>
#else
#include <time.h>
#endif
#endif

#include "openglheader.h"
#include "utilities.h"

double app_time, tic_time, toc_time;

#ifdef __linux__  /* Linux */
#ifdef USE_HIFRCLOCK
static struct timespec res, tick0, tick1, tick2;
#else
static struct tms      clk;
static clock_t         tick0, tick1, tick2;
static double          ticks_per_sec = 100.0;
#endif
#else             /* Windows */
#ifdef USE_HIFRCLOCK
static char            performance;
static double          frequency;
static uint64_t        tick0, tick1, tick2;
#else
static clock_t         tick0, tick1, tick2;
#endif
#endif

#ifdef __linux__  /* Linux */
#ifdef USE_HIFRCLOCK
#define TDIFF(t1,t0) \
  (double)(t1.tv_sec-t0.tv_sec) + 1.0e-9*(double)(t1.tv_nsec-t0.tv_nsec)
#else
#define TDIFF(t1,t0) (double)(t1-t0)/ticks_per_sec
#endif
#else             /* Windows */
#ifdef USE_HIFRCLOCK
#define TDIFF(t1,t0) (double)(t1-t0)/frequency
#else
#define TDIFF(t1,t0) (double)(t1-t0)/(double)CLOCKS_PER_SEC
#endif
#endif

void TimerInit ( void )
{
#ifdef __linux__  /* Linux */
#ifdef USE_HIFRCLOCK
  if ( clock_getres ( CLOCK_REALTIME, &res ) )
    ExitOnError ( "TimerInit" );
  clock_gettime ( CLOCK_REALTIME, &tick0 );
  tick1 = tick2 = tick0;
#else
  ticks_per_sec = (double)sysconf ( _SC_CLK_TCK );
  tick0 = tick1 = tick2 = times ( &clk );
#endif
#else             /* Windows */
#ifdef USE_HIFRCLOCK
  LARGE_INTEGER fv;

  if ( (performance = (QueryPerformanceFrequency ( &fv ) != 0) ) {
    frequency = (double)fv;
    QueryPerformanceValue ( &fv );
    tick0 = tick1 = tick2 = (uint_64_t)fv;
  }
  else {
    frequency = 1000.0;
    tick0 = tick1 = tick2 = (uint64_t)timeGetTime ();
  }
#else
  tick0 = tick1 = tick2 = clock ();
#endif
#endif
  app_time = toc_time = 0.0;
} /*TimerInit*/

double TimerTic ( void )
{
#ifdef __linux__  /* Linux */
#ifdef USE_HIFRCLOCK
  clock_gettime ( CLOCK_REALTIME, &tick1 );
  tick2 = tick1;
#else
  tick1 = tick2 = times ( &clk );
#endif
#else             /* Windows */
#ifdef USE_HIFRCLOCK
  LARGE_INTEGER tick;

  if ( performance ) {
    QueryPerformanceValue ( &tick );
    tick1 = tick2 = (uint64_t)tick;
  }
  else
    tick1 = tick2 = (uint64_t)timeGetTime ();
#else
  tick1 = tick2 = clock ();
#endif
#endif
  toc_time = 0.0;
  return app_time = tic_time = TDIFF(tick2,tick0);
} /*TimerTic*/

double TimerToc ( void )
{
#ifdef __linux__  /* Linux */
#ifdef USE_HIFRCLOCK
  clock_gettime ( CLOCK_REALTIME, &tick2 );
#else
  tick2 = times ( &clk );
#endif
#else             /* Windows */
#ifdef USE_HIFRCLOCK
  LARGE_INTEGER tick;

  if ( performance ) {
    QueryPerformanceValue ( &tick );
    tick2 = (uint64_t)tick;
  }
  else
    tick2 = (uint64_t)timeGetTime ();
#else
  tick2 = clock ();
#endif
#endif
  app_time = TDIFF(tick2,tick0);
  return toc_time = TDIFF(tick2,tick1);
} /*TimerToc*/

double TimerTocTic ( void )
{
#ifdef __linux__  /* Linux */
#ifdef USE_HIFRCLOCK
  struct timespec tick;

  tick = tick1;
  clock_gettime ( CLOCK_REALTIME, &tick1 );
  tick2 = tick1;
#else
  clock_t tick;

  tick = tick1;  tick1 = tick2 = times ( &clk );
#endif
#else             /* Windows */
#ifdef USE_HIFRCLOCK
  uint64_t tick;

  tick = tick1;
  if ( performance ) {
    QueryPerformanceValue ( &tick );
    tick1 = tick2 = (uint64_t)tick;
  }
  else
    tick1 = tick2 = (uint64_t)timeGetTime ();
#else
  clock_t tick;

  tick = tick1;  tick1 = tick2 = clock ();
#endif
#endif
  app_time = TDIFF ( tick2, tick0 );
  return toc_time = TDIFF ( tick2, tick );
} /*TimerTocTic*/

