#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "openglheader.h" /* najpierw ten */

#include "utilities.h"

GLuint tetra_vao, tetra_vbo[3],
       cube_vao, cube_vbo[3],
       octa_vao, octa_vbo[3],
       icos_vao, icos_vbo[3],
       dodec_vao, dodec_vbo[3];

void ConstructTetrahedronVAO ( void )
{
#define R 0.57735027
  static const GLfloat vertpos[4][3] =
    {{-R,-R,-R},{R,R,-R},{R,-R,R},{-R,R,R}};
  static const GLubyte vertcol[4][3] =
    {{255,0,0},{0,255,0},{0,0,255},{255,255,0}};
  static const GLubyte vertind[15] =
    {0,1,2,3,0,     /* lamana */
     1,3,2,0,       /* 2 odcinki */
     0,1,2,3,0,1};  /* tasma */

  glGenVertexArrays ( 1, &tetra_vao );
  glBindVertexArray ( tetra_vao );
  glGenBuffers ( 3, tetra_vbo );
  glBindBuffer ( GL_ARRAY_BUFFER, tetra_vbo[0] );
  glBufferData ( GL_ARRAY_BUFFER,
                 4*3*sizeof(GLfloat), vertpos, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 0 );
  glVertexAttribPointer ( 0, 3, GL_FLOAT, GL_FALSE,
                          3*sizeof(GLfloat), (GLvoid*)0 );
  glBindBuffer ( GL_ARRAY_BUFFER, tetra_vbo[1] );
  glBufferData ( GL_ARRAY_BUFFER,
                 4*3*sizeof(GLubyte), vertcol, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 1 );
  glVertexAttribPointer ( 1, 3, GL_UNSIGNED_BYTE, GL_TRUE,
                          3*sizeof(GLubyte), (GLvoid*)0 );
  glBindBuffer ( GL_ELEMENT_ARRAY_BUFFER, tetra_vbo[2] );
  glBufferData ( GL_ELEMENT_ARRAY_BUFFER,
                 15*sizeof(GLubyte), vertind, GL_STATIC_DRAW );
  ExitIfGLError ( "ConstructTetrahedronVAO" );
#undef R
} /*ConstructTetrahedronVAO*/

void ConstructCubeVAO ( void )
{
#define R 0.57735027
  static const GLfloat vertpos[8][3] =
    {{-R,-R,-R},{-R,R,-R},{-R,R,R},{-R,-R,R},{R,-R,-R},{R,R,-R},{R,R,R},{R,-R,R}};
  static const GLubyte vertcol[8][3] =
    {{255,0,0},{255,127,0},{255,255,0},{127,255,0},{0,255,0},{0,255,127},
     {0,255,255},{0,127,255}};
  static const GLubyte vertind[32] =
    {0,1,2,3,7,5,4,0,   /* lamana */
     0,3,5,6,0,4,2,6,   /* 4 odcinki */
     0,1,2,3,6,7,5,4,2,6,1,5,0,4,3,7}; /* 2 tasmy */

  glGenVertexArrays ( 1, &cube_vao );
  glBindVertexArray ( cube_vao );
  glGenBuffers ( 3, cube_vbo );
  glBindBuffer ( GL_ARRAY_BUFFER, cube_vbo[0] );
  glBufferData ( GL_ARRAY_BUFFER,
                 8*3*sizeof(GLfloat), vertpos, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 0 );
  glVertexAttribPointer ( 0, 3, GL_FLOAT, GL_FALSE,
                          3*sizeof(GLfloat), (GLvoid*)0 );
  glBindBuffer ( GL_ARRAY_BUFFER, cube_vbo[1] );
  glBufferData ( GL_ARRAY_BUFFER,
                 8*3*sizeof(GLubyte), vertcol, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 1 );
  glVertexAttribPointer ( 1, 3, GL_UNSIGNED_BYTE, GL_TRUE,
                          3*sizeof(GLubyte), (GLvoid*)0 );
  glBindBuffer ( GL_ELEMENT_ARRAY_BUFFER, cube_vbo[2] );
  glBufferData ( GL_ELEMENT_ARRAY_BUFFER,
                 32*sizeof(GLubyte), vertind, GL_STATIC_DRAW );
  ExitIfGLError ( "ConstructCubeVAO" );
#undef R
} /*ConstructCubeVAO*/

void ConstructOctahedronVAO ( void )
{
#define R 1.0
  static const GLfloat vertpos[6][3] =
    {{0,0,R},{-R,0,0},{0,-R,0},{R,0,0},{0,R,0},{0,0,-R}};
  static const GLubyte vertcol[6][3] =
    {{255,0,0},{255,127,0},{255,255,0},{127,255,0},{0,255,0},{0,255,127}};
  static const GLubyte vertind[25] =
    {0,1,2,3,4,1,5,3,0,2,5,4,0,  /* lamana */
     0,1,2,3,4,1,5,1,4,3,2,1};   /* 2 wachlarze */

  glGenVertexArrays ( 1, &octa_vao );
  glBindVertexArray ( octa_vao );
  glGenBuffers ( 3, octa_vbo );
  glBindBuffer ( GL_ARRAY_BUFFER, octa_vbo[0] );
  glBufferData ( GL_ARRAY_BUFFER,
                 6*3*sizeof(GLfloat), vertpos, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 0 );
  glVertexAttribPointer ( 0, 3, GL_FLOAT, GL_FALSE,
                          3*sizeof(GLfloat), (GLvoid*)0 );
  glBindBuffer ( GL_ARRAY_BUFFER, octa_vbo[1] );
  glBufferData ( GL_ARRAY_BUFFER,
                 6*3*sizeof(GLubyte), vertcol, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 1 );
  glVertexAttribPointer ( 1, 3, GL_UNSIGNED_BYTE, GL_TRUE,
                          3*sizeof(GLubyte), (GLvoid*)0 );
  glBindBuffer ( GL_ELEMENT_ARRAY_BUFFER, octa_vbo[2] );
  glBufferData ( GL_ELEMENT_ARRAY_BUFFER,
                 25*sizeof(GLubyte), vertind, GL_STATIC_DRAW );
  ExitIfGLError ( "ConstructOctahedronVAO" );
#undef R
} /*ConstructOctahedronVAO*/

void ConstructDodecahedronVAO ( void )
{
#define A 0.35682209
#define B 0.57735027
#define C 0.93417236
  static const GLfloat vertpos[20][3] =
    {{ 0.0, A,-C},{ 0.0,-A,-C},{-B,-B,-B},{-C, 0.0,-A},{-B, B,-B},
     { B, B,-B},{ B,-B,-B},{-A,-C, 0.0},{-C, 0.0, A},{-A, C, 0.0},
     { C, 0.0,-A},{ A,-C, 0.0},{-B,-B, B},{-B, B, B},{ A, C, 0.0},
     { C, 0.0, A},{ B,-B, B},{ 0.0,-A, C},{ 0.0, A, C},{ B, B, B}};
  static const GLubyte vertcol[20][3] =
    {{255,0,0},{255,127,0},{255,255,0},{127,255,0},{0,255,0},{0,255,127},
     {0,255,255},{0,127,255},{0,0,255},{127,0,255},{255,0,255},{255,0,127},
     {0,255,0},{0,255,127}, {0,255,255},{0,127,255},{0,0,255},{127,0,255},
     {255,0,255},{255,0,127}};
  static const GLubyte vertind[79] =
    {0,4,3,2,1,6,10,15,19,18,17,16,11,7,12,8,13,9,17,5,0, /* lamana */
     0,1,5,10,14,19,1,18,12,17, /* 5 odcinkow */
     0,1,2,3,4,5,9,14,5,10,6,1,  /* 4 wachlarze */
     7,2,1,6,11,16,17,12,8,3,2,
     13,9,14,19,18,17,12,8,3,4,9,
     15,19,14,5,10,6,11,16,17,18,19};

  glGenVertexArrays ( 1, &dodec_vao );
  glBindVertexArray ( dodec_vao );
  glGenBuffers ( 3, dodec_vbo );
  glBindBuffer ( GL_ARRAY_BUFFER, dodec_vbo[0] );
  glBufferData ( GL_ARRAY_BUFFER,
                 20*3*sizeof(GLfloat), vertpos, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 0 );
  glVertexAttribPointer ( 0, 3, GL_FLOAT, GL_FALSE,
                          3*sizeof(GLfloat), (GLvoid*)0 );
  glBindBuffer ( GL_ARRAY_BUFFER, dodec_vbo[1] );
  glBufferData ( GL_ARRAY_BUFFER,
                 20*3*sizeof(GLubyte), vertcol, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 1 );
  glVertexAttribPointer ( 1, 3, GL_UNSIGNED_BYTE, GL_TRUE,
                          3*sizeof(GLubyte), (GLvoid*)0 );
  glBindBuffer ( GL_ELEMENT_ARRAY_BUFFER, dodec_vbo[2] );
  glBufferData ( GL_ELEMENT_ARRAY_BUFFER,
                 79*sizeof(GLubyte), vertind, GL_STATIC_DRAW );
  ExitIfGLError ( "ConstructDodecahedronVAO" );
#undef A
#undef B
#undef C
} /*ConstructDodecahedronVAO*/

void ConstructIcosahedronVAO ( void )
{
#define A 0.52573115
#define B 0.85065085
  static const GLfloat vertpos[12][3] =
    {{ -A,0.0, -B},{  A,0.0, -B},{0.0, -B, -A},{ -B, -A,0.0},
     { -B,  A,0.0},{0.0,  B, -A},{  A,0.0,  B},{ -A,0.0,  B},
     {0.0, -B,  A},{  B, -A,0.0},{  B,  A,0.0},{0.0,  B,  A}};
  static const GLubyte vertcol[12][3] =
    {{255,0,0},{255,127,0},{255,255,0},{127,255,0},{0,255,0},{0,255,127},
     {0,255,255},{0,127,255},{0,0,255},{127,0,255},{255,0,255},{255,0,127}};
  static const GLubyte vertind[62] =
     { 0, 1, 2, 0, 3, 4, 0, 5, 1, 9, 2, 8, 3, /* lamana, od 0 */
       7, 4, 11, 5, 10, 9, 6, 8, 7, 6, 11, 7,
       1, 10, 6,                              /* lamana, od 25 */
       2, 3, 4, 5, 8, 9, 10, 11,              /* 4 odcinki, od 28 */
       0, 1, 2, 3, 4, 5, 1,                   /* wachlarz, od 36 */
       6, 7, 8, 9, 10, 11, 7,                 /* wachlarz, od 43 */
       1, 9, 2, 8, 3, 7, 4, 11, 5, 10, 1, 9}; /* tasma, od 50 */

  glGenVertexArrays ( 1, &icos_vao );
  glBindVertexArray ( icos_vao );
  glGenBuffers ( 3, icos_vbo );
  glBindBuffer ( GL_ARRAY_BUFFER, icos_vbo[0] );
  glBufferData ( GL_ARRAY_BUFFER,
                 12*3*sizeof(GLfloat), vertpos, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 0 );
  glVertexAttribPointer ( 0, 3, GL_FLOAT, GL_FALSE,
                          3*sizeof(GLfloat), (GLvoid*)0 );
  glBindBuffer ( GL_ARRAY_BUFFER, icos_vbo[1] );
  glBufferData ( GL_ARRAY_BUFFER,
                 12*3*sizeof(GLubyte), vertcol, GL_STATIC_DRAW );
  glEnableVertexAttribArray ( 1 );
  glVertexAttribPointer ( 1, 3, GL_UNSIGNED_BYTE, GL_TRUE,
                          3*sizeof(GLubyte), (GLvoid*)0 );
  glBindBuffer ( GL_ELEMENT_ARRAY_BUFFER, icos_vbo[2] );
  glBufferData ( GL_ELEMENT_ARRAY_BUFFER,
                 62*sizeof(GLubyte), vertind, GL_STATIC_DRAW );
  ExitIfGLError ( "ConstructIcosahedronVAO" );
#undef A
#undef B
} /*ConstructIcosahedronVAO*/

int main ( void )
{
#ifdef QQ
#define A 0.52573115
#define B 0.85065085
  static const double vertpos[12][3] =
    {{ -A,0.0, -B},{  A,0.0, -B},{0.0, -B, -A},{ -B, -A,0.0},
     { -B,  A,0.0},{0.0,  B, -A},{  A,0.0,  B},{ -A,0.0,  B},
     {0.0, -B,  A},{  B, -A,0.0},{  B,  A,0.0},{0.0,  B,  A}};
  GLubyte vv[20][3] =
    {{0,1,5},{0,2,1},{0,2,3},{0,3,4},{0,4,5},
     {1,5,10},{2,9,1,},{3,8,2},{4,7,3},{5,11,4},
     {1,9,10},{2,9,8},{3,8,7},{4,7,11},{5,11,10},
     {6,10,9},{6,8,9},{6,7,8},{6,11,7},{6,10,11}};
  int i, j;
  double s[3], t;

  for ( i = 0; i < 20; i++ ) {
    for ( j = 0; j < 3; j++ )
      s[j] = vertpos[vv[i][0]][j]+vertpos[vv[i][1]][j]+vertpos[vv[i][2]][j];
      t = sqrt ( s[0]*s[0]+s[1]*s[1]+s[2]*s[2] );
      printf ( "{%11.8f,%11.8f,%11.8f},\n", s[0]/t, s[1]/t, s[2]/t );
  }
#endif
  exit ( 0 );
} /*main*/

