#include <stdlib.h>
#include <string.h>
#include <stdio.h>    
#include <math.h>   
#include "openglheader.h"

#include "utilities.h"
#include "bezpatches3.h"

#define NBEZPATCHUOFFS 13

GLuint bezpbbp, cpbbp, cpibbp, txcbp;
GLint  bezpbsize;
static GLint bezpbofs[NBEZPATCHUOFFS];

const GLchar *UCPNames[] =
  { "CPoints", "CPoints.cp" };
const GLchar *UCPINames[] =
  { "CPIndices", "CPIndices.cpi" };
const GLchar *UBezPatchNames[NBEZPATCHUOFFS+1] =
  { "BezPatch", "BezPatch.npatches", "BezPatch.dim", "BezPatch.udeg",
    "BezPatch.vdeg", "BezPatch.stride_u", "BezPatch.stride_v",
    "BezPatch.stride_p", "BezPatch.stride_q", "BezPatch.nq",
    "BezPatch.use_ind", "BezPatch.Colour",
    "BezPatch.TessLevel", "BezPatch.BezNormals" };
const GLchar *UTexCoordNames[] =
  { "BezPatchTexCoord", "BezPatchTexCoord.txc" };

void GetAccessToBezPatchUniformBlocks ( GLuint program_id )
{
  GLint  size, ofs;

  GetAccessToUniformBlock ( program_id, 1, &UCPNames[0],
                            &size, &ofs, &cpbbp );
  GetAccessToUniformBlock ( program_id, 1, &UCPINames[0],
                            &size, &ofs, &cpibbp );
  GetAccessToUniformBlock ( program_id, NBEZPATCHUOFFS, &UBezPatchNames[0],
                            &bezpbsize, bezpbofs, &bezpbbp );
  GetAccessToUniformBlock ( program_id, 1, &UTexCoordNames[0],
                            &size, &ofs, &txcbp );
} /*GetAccessToBezPatchUniformBlocks*/

BezierPatchObjf* EnterBezierPatches ( GLint udeg, GLint vdeg, GLint dim,
                      GLint np, GLint nq, GLint ncp, const GLfloat *cp,
                      GLint stride_p, GLint stride_q, GLint stride_u, GLint stride_v,
                      const GLfloat *colour )
{
  BezierPatchObjf *bp;
  GLint           size;
  int             zero = 0, one = 1, tl = 10;

  if ( dim < 2 || dim > 4 || np*nq < 1 ||
       udeg < 1 || udeg > MAX_PATCH_DEGREE ||
       vdeg < 1 || vdeg > MAX_PATCH_DEGREE )
    return NULL;
  bp = malloc ( sizeof(BezierPatchObjf) );
  if ( bp ) {
    memset ( bp, 0, sizeof(BezierPatchObjf) );
    bp->udeg = udeg;  bp->vdeg = vdeg;  bp->dim = dim;
    bp->npatches = np*nq;
    bp->stride_p = stride_p;  bp->stride_q = stride_q;
    bp->stride_u = stride_u;  bp->stride_v = stride_v;

    bp->buf[0] = NewUniformBlockObject ( bezpbsize, bezpbbp );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[0], sizeof(GLint), &bp->npatches );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[1], sizeof(GLint), &bp->dim );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[2], sizeof(GLint), &bp->udeg );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[3], sizeof(GLint), &bp->vdeg );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[4], sizeof(GLint), &bp->stride_u );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[5], sizeof(GLint), &bp->stride_v );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[6], sizeof(GLint), &bp->stride_p );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[7], sizeof(GLint), &bp->stride_q );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[8], sizeof(GLint), &nq );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[9], sizeof(GLint), &zero );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[10], 3*sizeof(GLfloat), colour );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[11], sizeof(GLint), &tl );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[12], sizeof(GLint), &one );

    size = ncp*dim*sizeof(GLfloat);
    bp->buf[1] = NewUniformBlockObject ( size, cpbbp );
    glBufferSubData ( GL_UNIFORM_BUFFER, 0, size, cp );
    ExitIfGLError ( "EnterBezierPatches" );
    GenEmptyVAO ();
  }
  return bp;
} /*EnterBezierPatches*/

BezierPatchObjf* EnterBezierPatchesElem ( GLint udeg, GLint vdeg, GLint dim,
                      int npatches, int ncp,
                      const GLfloat *cp, const GLint *ind,
                      const GLfloat *colour )
{
  BezierPatchObjf *bp;
  GLint           size, one = 1, tl = 10;

  if ( dim < 2 || dim > 4 || npatches < 1 ||
       udeg < 1 || udeg > MAX_PATCH_DEGREE ||
       vdeg < 1 || vdeg > MAX_PATCH_DEGREE )
    return NULL;
  bp = malloc ( sizeof(BezierPatchObjf) );
  if ( bp ) {
    memset ( bp, 0, sizeof(BezierPatchObjf) );
    bp->udeg = udeg;  bp->vdeg = vdeg;  bp->dim = dim;
    bp->npatches = npatches;
    bp->buf[0] = NewUniformBlockObject ( bezpbsize, bezpbbp );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[0], sizeof(GLint), &bp->npatches );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[1], sizeof(GLint), &bp->dim );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[2], sizeof(GLint), &bp->udeg );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[3], sizeof(GLint), &bp->vdeg );
    bp->stride_p = (udeg+1)*(vdeg+1);
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[6], sizeof(GLint), &bp->stride_p );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[7], sizeof(GLint), &one );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[9], sizeof(GLint), &one );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[10], 3*sizeof(GLfloat), colour );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[11], sizeof(GLint), &tl );
    glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[12], sizeof(GLint), &one );

    size = ncp*dim*sizeof(GLfloat);
    bp->buf[1] = NewUniformBlockObject ( size, cpbbp );
    glBufferSubData ( GL_UNIFORM_BUFFER, 0, size, cp );

    size = (udeg+1)*(vdeg+1)*npatches*sizeof(GLint);
    bp->buf[2] = NewUniformBlockObject ( size, cpibbp );
    glBufferSubData ( GL_UNIFORM_BUFFER, 0, size, ind );
    ExitIfGLError ( "EnterBezierPatchesElem" );
    GenEmptyVAO ();
  }
  return bp;
} /*EnterBezierPatchesElem*/

void SetBezierPatchOptions ( BezierPatchObjf *bp, GLint normals, GLint tesslevel )
{
  glBindBuffer ( GL_UNIFORM_BUFFER, bp->buf[0] );
  glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[11], sizeof(GLint), &tesslevel );
  glBufferSubData ( GL_UNIFORM_BUFFER, bezpbofs[12], sizeof(GLint), &normals );
  ExitIfGLError ( "SetBezierPatchOptions" );
} /*SetBezierPatchOptions*/

void DrawBezierPatches ( BezierPatchObjf *bp )
{
  if ( bp ) {
    glBindBufferBase ( GL_UNIFORM_BUFFER, bezpbbp, bp->buf[0] );
    glBindBufferBase ( GL_UNIFORM_BUFFER, cpbbp, bp->buf[1] );
    if ( bp->buf[2] )
      glBindBufferBase ( GL_UNIFORM_BUFFER, cpibbp, bp->buf[2] );
    glBindVertexArray ( empty_vao );
    glPatchParameteri ( GL_PATCH_VERTICES, 1 );
    glDrawArraysInstanced ( GL_PATCHES, 0, 1, bp->npatches );
    glBindVertexArray ( 0 );
    ExitIfGLError ( "DrawBezierPatches" );
  }
} /*DrawBezierPatch*/

void DrawBezierNets ( BezierPatchObjf *bp )
{
  int nlines;

  if ( bp ) {
    glBindBufferBase ( GL_UNIFORM_BUFFER, bezpbbp, bp->buf[0] );
    glBindBufferBase ( GL_UNIFORM_BUFFER, cpbbp, bp->buf[1] );
    if ( bp->buf[2] )
      glBindBufferBase ( GL_UNIFORM_BUFFER, cpibbp, bp->buf[2] );
    glBindVertexArray ( empty_vao );
    nlines = 2*bp->udeg*bp->vdeg + bp->udeg + bp->vdeg;
    glDrawArraysInstanced ( GL_LINES, 0, 2, bp->npatches*nlines );
    glBindVertexArray ( 0 );
    ExitIfGLError ( "DrawBezierNets" );
  }
} /*DrawBezierNets*/

void DeleteBezierPatches ( BezierPatchObjf *bp )
{
  if ( bp ) {
    glDeleteBuffers ( 4, bp->buf );
    free ( bp );
  }
} /*DeleteBezierPatches*/
